  /*  rcfile.c - powershell
   *  Copyright (C) 1999  Matt Spong <spong@glue.umd.edu>
   *  Based from zterm.c by Michael Zucci from the libzvt docs
   *
   *  A big, puffy lotsa-shells-in-one app. 
   *
   *  This program is free software; you can redistribute it and/or modify
   *  it under the terms of the GNU General Public License as published by
   *  the Free Software Foundation; either version 2 of the License, or
   *  (at your option) any later version.
   *
   *  This program is distributed in the hope that it will be useful,
   *  but WITHOUT ANY WARRANTY; without even the implied warranty of
   *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   *  GNU General Public License for more details.
   *
   *  You should have received a copy of the GNU General Public License
   *  along with this program; if not, write to the Free Software
   *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   */

#include "powershell.h"

#define DEFAULT_RCFILE "# PowerShell v0.1\n\
#\n\
# Syntax:\n\
#\n\
# For each application you want to have a menu option, you need the\n\
# following entry:\n\
#\n\
# [app]\n\
# <tab name>\n\
# <command>\n\
# <menu title>\n\
#\n\
# If you want to define a custom menu for the application, you need the\n\
# following (this must come before the next app entry)\n\
#\n\
# [appmenu]\n\
# <menu title>\n\
# <menu item name> <accelerator> <keystroke>\n\
\n\
[app]\n\
tcsh\n\
/bin/tcsh\n\
New _tcsh tab\n\
\n\
[app]\n\
bash\n\
/bin/bash\n\
New _bash tab\n\
\n\
[app]\n\
pico\n\
/usr/bin/pico\n\
Run _pico\n\
\n\
[app]\n\
pine\n\
/usr/bin/pine\n\
Run pi_ne\n\
\n\
[app]\n\
emacs\n\
/usr/bin/emacs -nw\n\
Run _emacs\n\
\n\
[app]\n\
lynx\n\
/usr/bin/lynx\n\
Run _Lynx\n\
\n\
[options]\n\
http_handler /usr/bin/lynx\n\
ftp_handler /usr/bin/ncftp\n\
mail_handler /usr/bin/pine\n\
telnet_handler /usr/bin/telnet\n\
\n\
use_pixmap false\n\
transparent false\n\
shaded false\n\
"


int num_apps;

appentry * app_entries;

int InitRCFile ()
{
  FILE * rcfile;
  char * home_dir;
  char tmp[512];
  int a;
  unsigned char c;
  char buf[256];

  int pos;
  
  num_apps = 0;

  bzero (&Prefs, sizeof (Prefs));

  home_dir = getenv ("HOME");
  sprintf (tmp, "%s/.pshellrc", home_dir);
  rcfile = fopen (tmp, "r");
  
  if (!rcfile)
    {
      fprintf (stderr, "%s/.pshellrc does not exist; generating from defaults...\n", home_dir);
      rcfile = fopen (tmp, "w");
      if (!rcfile)
	{
	  fprintf (stderr, "Error opening %s/.pshellrc for writing\n", home_dir);
	  return -1;
	}
      fprintf (rcfile, DEFAULT_RCFILE);
      fclose (rcfile);
      rcfile = fopen (tmp, "r");
      if (!rcfile)
	{
	  fprintf (stderr, "Error opening %s/.pshellrc for reading\n", home_dir);
	  return -1;
	}
    }

  pos = -1;
  num_apps = 0;
  app_entries = (appentry *)malloc (sizeof(appentry));
  while (!feof (rcfile))
    {
      if (!strcmp (tmp, "[app]"))
	{
	  num_apps ++;
	  app_entries  = (appentry *)realloc (app_entries, sizeof(appentry) * num_apps);
	}

      a = 0;

      bzero (tmp, 511);

      while (((c = fgetc (rcfile))!='\n') && !feof(rcfile))
	{
	  tmp[a++] = c;
	}

      tmp[a] = '\0';

      if (tmp[0] != '#')
	{
	  if (!strcmp (tmp, "[app]"))
	  {
	    pos = 0;
	  }

	  if (strlen (tmp))
	    switch (pos)
	      {
	      case 0:
		pos++;
		break;
	      case 1:
		app_entries[num_apps-1].tab_title = (char *)malloc (strlen(tmp)+1);
		strcpy (app_entries[num_apps-1].tab_title, tmp);
		pos++;
		break;
	      case 2:
		app_entries[num_apps-1].command = (char *)malloc (strlen(tmp)+1);
		strcpy (app_entries[num_apps-1].command, tmp);
		pos++;
		break;
	      case 3:
		app_entries[num_apps-1].menu_item_title = 
		  (char *)malloc (strlen(tmp)+1);
		strcpy (app_entries[num_apps-1].menu_item_title, tmp);
		pos++;
		break;		
	      }
	  if (strstr (tmp, "http_handler"))
	    strcpy (Prefs.http_handler, tmp + 13);
	  if (strstr (tmp, "ftp_handler"))
	    strcpy (Prefs.ftp_handler, tmp + 12);
	  if (strstr (tmp, "mail_handler"))
	    strcpy (Prefs.mail_handler, tmp + 13);
	  if (strstr (tmp, "telnet_handler"))
	    strcpy (Prefs.telnet_handler, tmp + 15);
	  if (strstr (tmp, "bg_pixmap"))
	    strcpy (Prefs.bg_pixmap, tmp + 10); 
	  if (strstr (tmp, "use_pixmap"))
	    {
	      strcpy (buf, tmp + 11);
	      if (!strcmp (buf, "true"))
		Prefs.use_pixmap = 1;
	      else
		Prefs.use_pixmap = 0;
	    }
	  if (strstr (tmp, "transparent"))
	    {
	      strcpy (buf, tmp + 12);
	      if (!strcmp (buf, "true"))
		Prefs.transparent = 1;
	      else
		Prefs.transparent = 0;
	    }
	  if (strstr (tmp, "shaded"))
	    {
	      strcpy (buf, tmp + 7);
	      if (!strcmp (buf, "true"))
		Prefs.shaded = 1;
	      else
		Prefs.shaded = 0;
	    }
	  
	}
     


    }
  
  
  fclose (rcfile);
  free (home_dir);
  return 0;
}

int WriteRCFile()
{
  int a;
  FILE * rcfile;
  char * home_dir;
  char buf[512];

  home_dir = getenv ("HOME");
  sprintf (buf, "%s/.pshellrc", home_dir);
  
  rcfile = fopen (buf, "w");
  if (!rcfile)
    {
      fprintf (stderr, "Can't open %s for writing!\n", buf);
      return -1;
    }

  for (a=0;a<num_apps;a++)
    {
      fprintf (rcfile, "[app]\n");
      fprintf (rcfile, "%s\n", app_entries[a].tab_title);
      fprintf (rcfile, "%s\n", app_entries[a].command);
      fprintf (rcfile, "%s\n\n", app_entries[a].menu_item_title);
    }

  fprintf (rcfile, "[options]\n");
  fprintf (rcfile, "http_handler %s\n", Prefs.http_handler);
  fprintf (rcfile, "ftp_handler %s\n", Prefs.ftp_handler);
  fprintf (rcfile, "mail_handler %s\n", Prefs.mail_handler);
  fprintf (rcfile, "telnet_handler %s\n\n", Prefs.telnet_handler);

  fprintf (rcfile, "use_pixmap %s\n", Prefs.use_pixmap ? "true" : "false");
  fprintf (rcfile, "transparent %s\n", Prefs.transparent ? "true" : "false");
  fprintf (rcfile, "shaded %s\n", Prefs.shaded ? "true" : "false");
  if (strlen (Prefs.bg_pixmap))
    fprintf (rcfile, "bg_pixmap %s\n", Prefs.bg_pixmap);
  fclose (rcfile);
  return 0;
}
