  /*  term.c - powershell
   *  Copyright (C) 1999  Matt Spong <spong@glue.umd.edu>
   *  Based from zterm.c by Michael Zucci from the libzvt docs
   *
   *  A big, puffy lotsa-shells-in-one app. 
   *
   *  This program is free software; you can redistribute it and/or modify
   *  it under the terms of the GNU General Public License as published by
   *  the Free Software Foundation; either version 2 of the License, or
   *  (at your option) any later version.
   *
   *  This program is distributed in the hope that it will be useful,
   *  but WITHOUT ANY WARRANTY; without even the implied warranty of
   *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   *  GNU General Public License for more details.
   *
   *  You should have received a copy of the GNU General Public License
   *  along with this program; if not, write to the Free Software
   *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   */

#include "powershell.h"

GtkWidget * rd_dialog;
GtkWidget * rd_hbox1, * rd_label1, * rd_termname;
GtkWidget * nblabel;

void rd_ok_clicked (GtkButton * button, gpointer data)
{
  gtk_label_set (GTK_LABEL(nblabel),
		 gtk_entry_get_text (GTK_ENTRY(rd_termname)));
  gtk_widget_destroy (rd_dialog);
}

void RenameTerm()
{
  GtkWidget * ok;


  gchar ** curtermname;

  nblabel = gtk_notebook_get_tab_label (GTK_NOTEBOOK(notebook),
              gtk_notebook_get_nth_page (GTK_NOTEBOOK(notebook),
	      gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook))));

  gtk_label_get (GTK_LABEL(nblabel), (gchar **)(&curtermname));

  rd_dialog = gtk_dialog_new ();
  ok = gtk_button_new_with_label ("Ok");

  rd_hbox1 = gtk_hbox_new (TRUE, 5);
  rd_label1 = gtk_label_new ("New Tab Title: ");

  rd_termname = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY(rd_termname), (gchar*)curtermname);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(rd_dialog)->vbox), rd_hbox1, TRUE, TRUE, 0);

  gtk_box_pack_start (GTK_BOX(rd_hbox1), rd_label1, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(rd_hbox1), rd_termname, TRUE, TRUE, 0);
  


  gtk_container_add (GTK_CONTAINER(GTK_DIALOG(rd_dialog)->action_area), ok);
  gtk_signal_connect_object (GTK_OBJECT(ok), "clicked",
			     GTK_SIGNAL_FUNC (rd_ok_clicked),
			     GTK_OBJECT (rd_dialog));

  gtk_widget_show (rd_label1);
  gtk_widget_show (rd_termname);
  gtk_widget_show (rd_hbox1);
  gtk_widget_show (ok);

  gtk_window_set_modal (GTK_WINDOW(rd_dialog), TRUE);
  gtk_widget_show_all (rd_dialog);  
}

void PrepareArgs (char * command, char * args, char *** arg_array)
{
  int num;
  char * tmp;
  char * tmp2;

  char * cn;
  int a;
  char * args_cpy;

  tmp = strstr (command, "/") +1;

  if (!tmp)
    tmp = command;

  else
    {
      while (strstr (tmp, "/"))
	{
	  tmp = strstr (tmp, "/")+1;
	}
    }
  
  cn = (char *)malloc (strlen (tmp) + 1);
  
  strcpy (cn, tmp);

  if (strlen (args) == 0)
    {
      *arg_array = (char **)malloc (sizeof(char *) * 2);
      (*arg_array)[0] = (char *)malloc (strlen (cn) + 1);
      strcpy ((*arg_array)[0], cn);
      (*arg_array)[1] = NULL;
      return;
    }

  num = 1;

  args_cpy = (char *)malloc (strlen (args) + 1);
  strcpy (args_cpy, args);

  tmp = strstr (args_cpy, " ");
  if (tmp)
    {
      while (strstr (tmp, " "))
	{
	  tmp = strstr (tmp, " ") + 1;
	  num ++;
	}
    }

  *arg_array = (char **)malloc (sizeof(char *) * (2+num));
  (*arg_array)[0] = (char *)malloc (strlen (cn) + 1);
  strcpy ((*arg_array)[0], cn);

  tmp = args_cpy;

  for (a=0;a<num;a++)
    {
      tmp2 = strstr (tmp, " ");
      if (tmp2)
	tmp2[0] = '\0';

      (*arg_array)[a+1] = (char *)malloc (strlen(tmp) + 1);
      strcpy ((*arg_array)[a+1], tmp);
      if (tmp2)
	tmp2[0] = ' ';
      tmp = strstr (tmp, " ") +1;
    }

  (*arg_array)[num+1] = NULL;
}


GtkWidget * new_term (GtkWidget * notebook, gchar * command, gchar * args, gchar * title)
{
  GtkWidget *term, *scrollbar;
  enum { RIGHT, LEFT } scrollpos = LEFT;
  GString *shell, *name;
  GtkWidget * label;
  struct passwd *pw;
  gushort * colors, red[18], green[18], blue[18];
  int a,b;
  char tmp[512];
  char ** arg_array;

  struct stat * sanity_check;
  
  sanity_check = (struct stat *)malloc (sizeof(struct stat));
  terms_open ++;
  
  /* create terminal */
  
  term = zvt_term_new();
  
  if (!term)
    {
      fprintf (stderr, "Error creating new terminal\n");
      return NULL;
    }

  zvt_term_set_background (ZVT_TERM (term), Prefs.use_pixmap ? Prefs.bg_pixmap : NULL, 
			   Prefs.transparent, Prefs.shaded ? ZVT_BACKGROUND_SHADED : 0);
  
  zvt_term_set_font_name(ZVT_TERM (term), FONT);
  zvt_term_set_blink (ZVT_TERM (term), FALSE);
  zvt_term_set_bell (ZVT_TERM (term), TRUE);
  zvt_term_set_scrollback(ZVT_TERM (term), scrollbacklines);
  zvt_term_set_scroll_on_keystroke (ZVT_TERM (term), TRUE);
  zvt_term_set_scroll_on_output (ZVT_TERM (term), FALSE);



  zvt_term_set_wordclass (ZVT_TERM (term), "-A-Za-z0-9/_:.,?+%=");
  zvt_term_set_del_key_swap (ZVT_TERM (term), TRUE);
  
  PrepareArgs (command, args, &arg_array);

  /* fork the shell/program */
  switch (zvt_term_forkpty(ZVT_TERM (term), ZVT_TERM_DO_UTMP_LOG |  
			   ZVT_TERM_DO_WTMP_LOG))
    {
    case -1:
      perror("ERROR: unable to fork:");
      return NULL;
      break;
      
    case 0:
      if (stat (command, sanity_check))
	{
	  execle ("/bin/sh", "sh", "-c", "echo The specified program does not exist ; sleep 3", NULL, env_copy);
	  
	} 
      
      if (!(sanity_check->st_mode & S_IXUSR))
	{
	  execle ("/bin/sh", "sh", "-c", "echo Permission denied ; sleep 3", NULL, env_copy);

	}

      
      execve (command, arg_array, env_copy);
      break;
      
    default:
      break;
    }

  label = gtk_label_new (title);
  gtk_widget_show (GTK_WIDGET(term));
  gtk_widget_show (GTK_WIDGET(label));
 
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), term, label);

  
  gtk_signal_connect (GTK_OBJECT (term),
		      "child_died",
		      GTK_SIGNAL_FUNC (child_died_event),
		      NULL);
  
  gtk_signal_connect (GTK_OBJECT (term),
		      "destroy",
		      GTK_SIGNAL_FUNC (term_died_event),
		      NULL);
  
  gtk_signal_connect (GTK_OBJECT (term),
		      "title_changed",
		      GTK_SIGNAL_FUNC (title_changed_event),
		      NULL);
  
  gtk_signal_connect_after (GTK_OBJECT (term),
			    "realize",
			    GTK_SIGNAL_FUNC (set_hints),
			    term);
  gtk_signal_connect (GTK_OBJECT (window),
		      "key-press-event",
		      GTK_SIGNAL_FUNC (term_key_event),
		      NULL);

  gtk_signal_connect (GTK_OBJECT (term),
		      "button_press_event",
		      GTK_SIGNAL_FUNC (term_click_event),
		      NULL);

  ZVT_TERM(term)->colors[16] = 0xffffffff;
  ZVT_TERM(term)->colors[17] = 0x0;
  
  zvt_term_match_add (ZVT_TERM(term), "http\\:\\/\\/([a-z]|[A-Z]|[0-9]|\\_|\\.|\\&|\\%|\\@|\\?|\\-|\\~|\\/|\\=|\\+|\\:)*", VTATTR_UNDERLINE, "http");
  zvt_term_match_add (ZVT_TERM(term), "ftp\\:\\/\\/([a-z]|[A-Z]|[0-9]|\\_|\\.|\\&|\\%|\\@|\\?|\\-|\\~|\\/|\\=|\\+|\\:)*", VTATTR_UNDERLINE, "ftp");
  zvt_term_match_add (ZVT_TERM(term), "([a-z]|[A-Z]|[0-9]|\\_|\\.|\\-|\\+)*\\@([a-z]|[A-Z]|[0-9]|\\_|\\.|\\-|\\+)*", VTATTR_UNDERLINE, "mailto");
  zvt_term_match_add (ZVT_TERM(term), "telnet\\:\\/\\/([a-z]|[A-Z]|[0-9]|\\_|\\.|\\&|\\%|\\@|\\?|\\-|\\~|\\/|\\=|\\+|\\:)*", VTATTR_UNDERLINE, "telnet");



  gtk_notebook_set_page (GTK_NOTEBOOK(notebook), -1);
  return term;
}







